package mhtree;

import cz.muni.fi.disa.similarityoperators.cover.AbstractRepresentation.PrecomputedDistances;
import messif.buckets.BucketStorageException;
import messif.buckets.LocalBucket;
import messif.objects.LocalAbstractObject;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 * Represents a leaf node in MH-Tree.
 */

public class LeafNode extends Node implements Serializable {

    /**
     * Serialization ID
     */
    private static final long serialVersionUID = 1L;

    /**
     * Bucket for storing objects of the MH-Tree.
     */
    private final LocalBucket bucket;

    protected LeafNode(PrecomputedDistances distances, LocalBucket bucket, InsertType insertType, ObjectToNodeDistance objectToNodeDistance) throws BucketStorageException {
        super(distances, insertType, objectToNodeDistance);

        this.bucket = bucket;
        this.bucket.addObjects(distances.getObjects());
    }

    /**
     * Adds {@code object} to the node's bucket.
     *
     * @param object object to be added
     * @throws BucketStorageException addition of object into bucket exception
     */
    protected void addObject(LocalAbstractObject object) throws BucketStorageException {
        addObject(object, null);
    }

    protected void addObject(LocalAbstractObject object, PrecomputedDistances distances) throws BucketStorageException {
        bucket.addObject(object);

        addObjectIntoHull(object, distances);
    }

    /**
     * Returns a list of objects in node's bucket.
     *
     * @return a list of objects in node's bucket
     */
    public List<LocalAbstractObject> getObjects() {
        List<LocalAbstractObject> objects = new ArrayList<>(bucket.getObjectCount());

        bucket
                .getAllObjects()
                .forEachRemaining(objects::add);

        return objects;
    }

    /**
     * Returns the number of objects stored in node's bucket.
     *
     * @return the number of objects stored in node's bucket
     */
    protected int getObjectCount() {
        return bucket.getObjectCount();
    }

    /**
     * Returns the height of this node.
     *
     * @return the height of this node
     */
    protected int getHeight() {
        return 0;
    }

    /**
     * Adds this node into {@code nodes}.
     *
     * @param nodes list of nodes
     */
    protected void gatherNodes(List<Node> nodes) {
        nodes.add(this);
    }

    /**
     * Adds this node into {@code leafNodes}.
     *
     * @param leafNodes list of leaf nodes
     */
    protected void gatherLeafNodes(List<LeafNode> leafNodes) {
        leafNodes.add(this);
    }
}
